import {Component, ElementRef, OnInit, ViewChild} from '@angular/core';
import {TaskService} from '../services/task.service';
import {LinkService} from '../services/link.service';
import {Task} from '../models/task';
import {Link} from '../models/link';

import 'dhtmlx-gantt';
import {} from '@types/dhtmlxgantt';

@Component({
    selector: 'gantt',
    styleUrls: ['gantt.component.css'],
    providers: [TaskService, LinkService],
    templateUrl: 'gantt.component.html'
})
export class GanttComponent implements OnInit {
    @ViewChild('gantt_here') ganttContainer: ElementRef;

    constructor(private taskService: TaskService, private linkService: LinkService) {
    }

    ngOnInit() {


        gantt.config.xml_date = '%Y-%m-%d %H:%i';

        gantt.init(this.ganttContainer.nativeElement);

        gantt.attachEvent('onAfterTaskAdd', (id, item) => {
            this.taskService.insert(this.serializeTask(item, true))
                .then((response) => {
                    if (response.id != id) {
                        gantt.changeTaskId(id, response.id);
                    }
                });
        });

        gantt.attachEvent('onAfterTaskUpdate', (id, item) => {
            this.taskService.update(this.serializeTask(item));
        });

        gantt.attachEvent('onAfterTaskDelete', (id) => {
            this.taskService.remove(id);
        });

        gantt.attachEvent('onAfterLinkAdd', (id, item) => {
            this.linkService.insert(this.serializeLink(item, true))
                .then((response) => {
                    if (response.id != id) {
                        gantt.changeLinkId(id, response.id);
                    }
                });
        });

        gantt.attachEvent('onAfterLinkUpdate', (id, item) => {
            this.linkService.update(this.serializeLink(item));
        });

        gantt.attachEvent('onAfterLinkDelete', (id) => {
            this.linkService.remove(id);
        });


// custom lightbox section \/\/
        let taskId = null;

        gantt.showLightbox = function (id) {
            taskId = id;
            const task = gantt.getTask(id);

            const textInput = getInput('description');
            getForm().style.display = 'block';
            getModalOverlay().style.display = 'block';

            textInput.focus();
            textInput.value = task.text;

            getButton('save').onclick = save;
            getButton('cancel').onclick = cancel;
            getButton('remove').onclick = remove;

        };


        gantt.hideLightbox = function () {
            getForm().style.display = 'none';
            getModalOverlay().style.display = 'none';
            taskId = null;
        };

        function getElement(id) {
            return document.getElementById(id);
        }

        function getModalOverlay() {
            return getElement('modal-overlay');
        }

        function getForm() {
            return getElement('my-form');
        }
        function getInput(id): HTMLInputElement {
            return document.getElementById(id) as HTMLInputElement;
        }

        function getButton(name): HTMLInputElement {
            return getForm().querySelector(`[name="${name}"]`) as HTMLInputElement;
        }

        function save() {
            const task = gantt.getTask(taskId);

            task.text = getInput('description').value;

            if (task.$new) {
                gantt.addTask(task, task.parent);
                task.$new = false;
            }
            else {
                gantt.updateTask(task.id);
            }
            gantt.hideLightbox();
        }

        function cancel() {
            const task = gantt.getTask(taskId);
            console.log('cancel', task);

            if (task.$new) {
                gantt.deleteTask(task.id);
            }

            gantt.hideLightbox();
        }

        function remove() {
            gantt.deleteTask(taskId);
            gantt.hideLightbox();
        }

// custom lightbox section /\/\

        Promise.all([this.taskService.get(), this.linkService.get()])
            .then(([data, links]) => {
                gantt.parse({data, links});
            });


    }

    private serializeTask(data: any, insert: boolean = false): Task {
        return this.serializeItem(data, insert) as Task;
    }

    private serializeLink(data: any, insert: boolean = false): Link {
        return this.serializeItem(data, insert) as Link;
    }

    private serializeItem(data: any, insert: boolean): any {
        var result = {};

        for (let i in data) {
            if (i.charAt(0) == '$' || i.charAt(0) == '_') continue;
            if (insert && i == 'id') continue;
            if (data[i] instanceof Date) {
                result[i] = gantt.templates.xml_format(data[i]);
            }
            else {
                result[i] = data[i];
            }
        }

        return result;
    }
}
